'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

/**
 * VueModules
 *
 * VueModules is a Vue plugin that tries to achieve some modularization of pages
 * and components
 */
var VueModules = {
  version: '0.0.0',

  /** @function
   * @name install
   * @param {Vue} Vue
   * @param {object} options Customizations of the plugin
   * @example Vue.use(VueModules, options)
   */
  install: function install(Vue, _ref) {
    var app = _ref.app,
        router = _ref.router,
        store = _ref.store,
        _ref$modules = _ref.modules,
        modules = _ref$modules === undefined ? [] : _ref$modules,
        _ref$beforeRoutePush = _ref.beforeRoutePush,
        beforeRoutePush = _ref$beforeRoutePush === undefined ? function () {} : _ref$beforeRoutePush;


    /** @function
     * @name registerModule
     * @param {object} options Customizations for module
     * @example Vue.registerModule({ resource, module })
     */
    Vue.registerModule = function (_ref2) {
      var resource = _ref2.resource,
          _ref2$module = _ref2.module,
          module = _ref2$module === undefined ? false : _ref2$module,
          _ref2$route = _ref2.route,
          route = _ref2$route === undefined ? false : _ref2$route,
          _ref2$custom = _ref2.custom,
          custom = _ref2$custom === undefined ? {} : _ref2$custom;

      var routes = [];

      // vuex registerModule
      store && store.registerModule(resource, module.store());

      if (router) {
        module.routes.forEach(function (_route) {
          return routeHandler(_route, {
            app: app,
            resource: resource,
            routes: routes,
            alias: route.alias,
            custom: custom,
            beforeRoutePush: beforeRoutePush
          });
        });

        // vue-router addRoutes
        router.addRoutes(routes);
      }
    };

    modules.length > 0 && modules.forEach(Vue.registerModule);
  }
};

/**
 * Route Handler
 *
 * Handles a route
 */
var routeHandler = function routeHandler(_route, _ref3) {
  var app = _ref3.app,
      resource = _ref3.resource,
      _ref3$routes = _ref3.routes,
      routes = _ref3$routes === undefined ? [] : _ref3$routes,
      _ref3$alias = _ref3.alias,
      alias = _ref3$alias === undefined ? false : _ref3$alias,
      _ref3$custom$routes = _ref3.custom.routes,
      customRoutes = _ref3$custom$routes === undefined ? [] : _ref3$custom$routes,
      beforeRoutePush = _ref3.beforeRoutePush;


  /**
   * Override a page/route
   *
   * In the example the route named as "user" will load the "MyCustomUserPage"
   * instead of the defined in the module
   *
   * @example
   * customRoutes = [
   *   { name: 'user', component: require('./MyCustomUserPage') }
   * ]
   */
  if (customRoutes.length > 0) {
    if (_route.name === customRoutes[0].name) {
      customRoutes[0].path = _route.path;
      _route = customRoutes[0];
    }
  }

  /**
   * Adds a alias for a route
   *
   * In the example the route "/usuario" is a duplication of the route "/user"
   *
   * @example route.alias = [ '/user=/usuario' ]
   */
  if (alias) {
    alias.forEach(function (item) {
      var _item$split = item.split('='),
          _item$split2 = _slicedToArray(_item$split, 2),
          match = _item$split2[0],
          alias = _item$split2[1];

      if (_route.path === match) {
        var path = alias;

        // module "base" does not need prefix
        if (resource !== 'base') path = '/' + resource + path;

        routes.push(Object.assign({}, _route, { path: path }));
      }
    });
  }

  // module "base" does not need prefix
  if (resource !== 'base') _route.path = '/' + resource + _route.path;

  var module = resource;
  var page = _route.name;

  beforeRoutePush(app, module, page);

  routes.push(_route);
};

exports.default = VueModules;

//# sourceMappingURL=index.js.map